// Persistence of Vision Ray Tracer Scene Description File
// File: snow_ex.pov
// Vers: 3.6.1
// Desc: Topview of Tree for Surcoat-Macro 
// Date: 09.12.04 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak - www.NoLights.de
// Last Update: 07.01.05 (dd.mm.yy)

//Required Files
// "surcoat.inc" - The Surcoat-Macros
// "sc2snow.inc" - Converts Surcoat-Data to Snow-Data
// "arial.ttf"   - TrueTypeFont to add some description text

//Description
// This scene will sample a test object and show two sorting-macros at work.

// 1.
// In the the upper left corner is the sorted surcoat data.
// 2.
// In the upper right corner, method 0 is demonstrated. Notice the rather linear
// falloff of the stacking effect.
// 3.
// In the lower left corner, method 1 is demonstrated. The stacking-falloff is
// smoother and much more realistic.
// 4.
// In the lower right corner, the results from method 1 were slightly turbulenced
// to achieve a more "natural" look.

//
/*Commandline
+AM2 +A0.1 +R2 +W800 +H640
*/

#include "surcoat.inc"
#include "sc2snow.inc"

global_settings{assumed_gamma 1}
#local Size = 25;
camera{
  orthographic
  location <0,0,-20>
  up y*Size
  right x*Size*1.33
  look_at <0,0,-10>
}

//Sunlight
light_source{<0,-1500,0>
  rgb 1
  rotate <125,42,0>
  }

//Backlight
light_source{
  <500,350,500>
  rgb <.05,.05,.3>
  shadowless
  parallel point_at <0,0,0>
}

//CSG-Object for tests
#declare Surcoat_Topview = pigment{image_map{png "example_1.png"}}

#declare Surcoat_Debug = on;
#declare Surcoat_Seed = 211083;

#declare Surcoat_CellResolution = 3;
#declare Surcoat_CellSamples = 15;

#declare Surcoat_Sample_Offset = .0001;
#declare Surcoat_Sample_Direction = -y-x+z;
#declare Surcoat_Sample_MaxDirectionDeviation = 90;
#declare Surcoat_Sample_OverrideDirection = 0;


#declare Test_Object = 
  merge{
    box{<-3,0,-3><3,1,3>}
    cylinder{<-2.5,0,-2.5><-2.5,4.01,-2.5>,.5}
    box{<-3,4,-3><3,5,3>}
    sphere{y*6,1}
    cylinder{y*4.5,y*6,1}
    pigment{rgb x}
  }    

//Create initial samples on surface
Surcoat_CreateSamples(Test_Object)
Surcoat_PrepareSorting()
Surcoat_KeepAngled(y,60)
Surcoat_RemoveRoofed(Test_Object,3.2,y,3,.65,45)
Surcoat_SaveSorted("sc_roof.tmp")

//Coverting Surcoat to Snowdata
//=============================

#declare Surcoat_Snow_Seed = 260981;
//"Sky-Vector" for the stacking
#declare Surcoat_Snow_Sky = y;
//Min and Max Radius for a snow-particle
#declare Surcoat_Snow_MinRadius = .1;
#declare Surcoat_Snow_MaxRadius = .3;
//Max-Stackingheight for a snow-particle
#declare Surcoat_Snow_MaxHeight = 1.5;
//Stackheight*WallInfluence is the distance until which snow-particles may stack
#declare Surcoat_Snow_WallInfluence = 2;
//Samples to determine the radius
#declare Surcoat_Snow_Samples_Radius = 10;
//Samples to determine proximity to walls
#declare Surcoat_Snow_Samples_Hemisphere = 50;
//After how many samples to debug a progression note
//(default is 0 and thus, "off")
#declare Surcoat_Snow_DebugSampleCount = 1000;

//Generating snow with Method 0
#declare Surcoat_Snow_StackMethod = 0;
Surcoat2Snowfile("sc_roof.tmp","sc_0.tmp",Test_Object)

//Generating snow with Method 1
#declare Surcoat_Snow_StackMethod = 1;
Surcoat2Snowfile("sc_roof.tmp","sc_1.tmp",Test_Object)

//Modifying the results of Method 1 for some more "natural" look
#declare Surcoat_Snow_StackMethod = 1;

#macro Snowdata2Float_HeightAdd(_Position,_Normal,_Radius,_Height)
  //Calculate a float from the given data and declare it as
  // "Return". Default macro returns 0
  #local Return = max(min(vlength(vturbulence(2,.6,9,_Position*.25))-.5,1),0)*.5;
  //At the end of the macro, return "Return", for simplicity's sake,
  //just keep this line. :o)
  (Return)
#end
//Macro to generate multiplier for the height
#macro Snowdata2Float_HeightMod(_Position,_Normal,_Radius,_Height)
  //Calculate a float from the given data and declare it as
  // "Return". Default macro returns 1
  #local Return = vlength(vturbulence(2,.6,9,_Position*.25+<17,4,12>))*1.5-.35;
  //At the end of the macro, return "Return", for simplicity's sake,
  //just keep this line. :o)
  (Return)
#end
Snowfile_Modify("sc_1.tmp","sc_1mod.tmp")

//This resets the Modifying macros to default
Surcoat2Snow_Defaults()


//Display the samples
union{
  Surcoat_Spheres("sc_roof.tmp",.1)
  object{Test_Object}
  pigment{rgb 1}
  rotate y*45 rotate x*-25
  translate <-7,5,0>
}

union{
  Snowfile_Display("sc_0.tmp",.5,.75,1,1)
  object{Test_Object}
  pigment{rgb 1}
  rotate y*45 rotate x*-25
  translate <7,5,0>
}

union{
  Snowfile_Display("sc_1.tmp",.5,.75,1,1)
  object{Test_Object}
  pigment{rgb 1}
  rotate y*45 rotate x*-25
  translate <-7,-7,0>
}

union{
  Snowfile_Display("sc_1mod.tmp",.5,.75,1,1)
  object{Test_Object}
  pigment{rgb 1}
  rotate y*45 rotate x*-25
  translate <7,-7,0>
}

//Description text
text{
  ttf "arial.ttf" "Surcoat-sorted" .2,0
  pigment{rgb 1}
  finish{ambient 1 diffuse 0}
  scale 2
  translate <-14,1,0>
}

text{
  ttf "arial.ttf" "Method 0" .2,0
  pigment{rgb 1}
  finish{ambient 1 diffuse 0}
  scale 2
  translate <3,1,0>
}

text{
  ttf "arial.ttf" "Method 1" .2,0
  pigment{rgb 1}
  finish{ambient 1 diffuse 0}
  scale 2
  translate <-11,-11,0>
}

text{
  ttf "arial.ttf" "M1-modified" .2,0
  pigment{rgb 1}
  finish{ambient 1 diffuse 0}
  scale 2
  translate <2,-11,0>
}

